#!/usr/bin/perl

use strict;
require Utils;

my $TEST_CASE_DIR="cases";
my $BINARY_NAME = "KeywordHunter";
my $ERROR_EXIT_CODE=256;
my $SUCCESS_EXIT_CODE=0;

if ( ! -f $BINARY_NAME) {
  die "Unable to find the binary file $BINARY_NAME in the current directory.\n";
}

# Create the answer files for the test cases
my @lCaseFiles = <$TEST_CASE_DIR/CASE*.txt>;
my $lCaseFile;
foreach $lCaseFile (@lCaseFiles) {
  CreateCaseAnswer($lCaseFile);
}

sub CreateCaseAnswer() {
  my $lCaseFile = shift(@_);
  print "Creating Case Answer : $lCaseFile\n";

  my $lCaseAnswerFile = "$lCaseFile.answer";
  if (-f $lCaseAnswerFile ) {
    print "Ignoring case $lCaseFile as answer already exists\n";
    return;
  }

  #Extract the details of the case.
  my %lCaseDetails = readConfigFile ($lCaseFile);
  my $lStartUrl = $lCaseDetails{"StartUrl"};
  my $lLevel = $lCaseDetails{"Level"};
  my $lSearchKeyword = $lCaseDetails{"SearchKeyword"};
  my $lOutputDir = $lCaseDetails{"OutputDir"};

  if (defined($lOutputDir)) {
    `rm -rf $lOutputDir`;
    mkdir ($lOutputDir) || die "Unable to create directory $lOutputDir\n";
  }

  if (  !defined($lStartUrl) ||
        !defined($lLevel) ||
        !defined($lSearchKeyword)
     ) {
    die "Invalid case file $lCaseFile.\n";
  }
  
  my $lTmpCmd = "./$BINARY_NAME $lStartUrl $lSearchKeyword $lOutputDir > $lCaseAnswerFile";
  `$lTmpCmd`;
  if ($? != $SUCCESS_EXIT_CODE) {
    die "FAIL: $lTmpCmd returned exit code $?, expected $SUCCESS_EXIT_CODE.  Unable to create answer for case $lCaseFile\n";
  }
}
