#!/usr/local/bin/perl

# @author Dilip Antony Joseph (dilip@cs.berekeley.edu)

use strict;
require Utils;

use File::Basename;

# THis flag must be set to 0 if Keyword hunter is actually run.  If it is 1, we assume that the Keyword hunter output is already present.
my $gNoRunFlag = 0;


my $VERSION = "0.3";

my $TEST_CASE_DIR="cases";
my $BINARY_NAME = "KeywordHunter";
my $DUMP_DIR="cases/";
my $ERROR_EXIT_CODE=256;
my $SUCCESS_EXIT_CODE=0;

my $gLateDays = 0;
if ($#ARGV + 1 >= 1) {
  $gLateDays = $ARGV[0];
}

if ( ! -f $BINARY_NAME) {
  die "Unable to find the binary file $BINARY_NAME in the current directory.\n";
}

print "\n\n";
print "Project 2 Evaluator version $VERSION\n";

my $MaxPoints = 0;
my $TotalPoints = 0;

my $gMaxFoundLinePoints = 0;
my $gMaxValidLinkTraversalPoints = 0;
my $gMaxIgnoredLinkTraversalPoints = 0;
my $gMaxSavedOutputPoints = 0;
my $gMaxCmdLineArgsPoints = 0;

my $lTmpTotalPoints;
my $lTmpMaxPoints;


($lTmpTotalPoints, $lTmpMaxPoints) = Test_CmdLineArgs();
$TotalPoints += $lTmpTotalPoints;
$MaxPoints += $lTmpMaxPoints;
$gMaxCmdLineArgsPoints = $lTmpMaxPoints;

# Run the test cases
my @lCaseFiles = <$TEST_CASE_DIR/CASE*.txt>;
my $lCaseFile;
foreach $lCaseFile (@lCaseFiles) {
  ($lTmpTotalPoints, $lTmpMaxPoints) = RunCase($lCaseFile);
  $TotalPoints += $lTmpTotalPoints;
  $MaxPoints += $lTmpMaxPoints;
}

print "\n\n";
print "============================================================================================\n";
print "Report\n";
print "######\n\n";

#print "\nNoRunFlag = $gNoRunFlag\n";

print "MaxCmdLineArgsPoints \t\t\t= $gMaxCmdLineArgsPoints\n";
print "MaxFoundLinePoints \t\t\t= $gMaxFoundLinePoints\n";
print "MaxValidLinkTraversalPoints \t\t= $gMaxValidLinkTraversalPoints\n";
print "MaxIgnoredLinkTraversalPoints \t\t= $gMaxIgnoredLinkTraversalPoints\n";
print "MaxSavedOutputPoints \t\t\t= $gMaxSavedOutputPoints\n";
print "Max Total \t\t\t\t= $MaxPoints\n";

print "\n\n";
print "#######################################################\n";
print "Late by     : $gLateDays day(s)\n";
my $lGrandTotal = 0;
my $lLateMsg = "";
if ($gLateDays == 0) {
  $lGrandTotal = $TotalPoints;
} elsif ($gLateDays == 1) {
  $lLateMsg = "(90 % of $TotalPoints)";
  $lGrandTotal = 0.9 * $TotalPoints;
} elsif ($gLateDays == 2) {
  $lLateMsg = "(80 % of $TotalPoints)";
  $lGrandTotal = 0.8 * $TotalPoints;
} elsif ($gLateDays == 3) {
  $lLateMsg = "(60 % of $TotalPoints)";
  $lGrandTotal = 0.6 * $TotalPoints;
} else {
  $lGrandTotal = 0;
}
$lGrandTotal = int ($lGrandTotal + 0.5);
print "Grand Total=[[[$lGrandTotal]]] (out of $MaxPoints) $lLateMsg\n\n";




sub Test_CmdLineArgs() {
  my $lTestName = "CmdLineArgs";

  my $lDumpFile = "$DUMP_DIR/proj2_dump.txt";

  my $lMaxPoints = 2;
  my $lPoints = 0;

  `./$BINARY_NAME > $lDumpFile`;
  if ($? == $ERROR_EXIT_CODE) {
    $lPoints = $lPoints + 1;
  }
 
  system("./$BINARY_NAME http://danjo.cs.berkeley.edu/index.html > $lDumpFile");
  if ($? == $ERROR_EXIT_CODE) {
    $lPoints ++;
  }
 
  if ($lPoints > $lMaxPoints) {
    die "Max Points exceeded for $lTestName\n";
  }

  print "\$\$\$ TestResult: CmdLineArgs: $lPoints (out of $lMaxPoints)\n";

  return ($lPoints, $lMaxPoints);
 
}


sub RunCase() {
  

  my $lCaseFile = shift(@_);
  print "\n\n";
  print "-----------------------------------------------------------------------------\n";
  print ">>> Running Case : $lCaseFile\n";

  my $lPoints = 0;

  my $lCaseAnswerFile = "$lCaseFile.answer";
  if (!-f $lCaseAnswerFile) {
    die "Unable to find the answer file $lCaseAnswerFile.\n";
  }

  #Extract the details of the case.
  my %lCaseDetails = readConfigFile ($lCaseFile);
  my $lStartUrl = $lCaseDetails{"StartUrl"};
  my $lLevel = $lCaseDetails{"Level"};
  my $lSearchKeyword = $lCaseDetails{"SearchKeyword"};
  my $lDesc = $lCaseDetails{"Description"};
  my $lMaxPoints = $lCaseDetails{"MaxPoints"};
  my $lOutputDir = $lCaseDetails{"OutputDir"};
  my $lTestOutputDir = "";
  
  if (defined($lOutputDir)) {
    $lTestOutputDir = "$lOutputDir.out";
    if ($gNoRunFlag == 1) {
      if (!-d $lTestOutputDir) {
        die "NoRunFlag set.  But directory html output directory $lTestOutputDir not found.\n";
      }
    } else {
      if ( -d $lTestOutputDir) {
        `rm -rf $lTestOutputDir`;
      }
      mkdir ($lTestOutputDir) || die "Unable to create directory $lTestOutputDir\n";
    }
  } 

  my $lDoFoundLineTest = $lCaseDetails{"DoFoundLineTest"};
  my $lDoValidLinkTraversalTest = $lCaseDetails{"DoValidLinkTraversalTest"};
  my $lDoIgnoredLinkTraversalTest = $lCaseDetails{"DoIgnoredLinkTraversalTest"};

  
  if (  !defined($lStartUrl) ||
        !defined($lLevel) ||
        !defined($lSearchKeyword) ||
        !defined($lMaxPoints) 
     ) {
    die "Invalid case file $lCaseFile.\n";
  }
  
  if (defined($lDesc)) {
    print "Description: $lDesc\n\n";
  }
  my $lCaseTmpFile = "$lCaseFile.out";
  if ( $gNoRunFlag != 1) {
    if (-f $lCaseTmpFile) {
      unlink($lCaseTmpFile) || die "Unable to delete existing case output file $lCaseTmpFile\n";
    }
    my $lTmpCmd = "./$BINARY_NAME $lStartUrl $lSearchKeyword $lTestOutputDir > $lCaseTmpFile";
    `$lTmpCmd`;
    if ($? != $SUCCESS_EXIT_CODE) {
      print "UNEXPECTED_RETURN_VALUE: $lTmpCmd returned exit code $?, expected $SUCCESS_EXIT_CODE\n";
    }

  } else {
    if (!-f $lCaseTmpFile) {
      die "NoRunFlag set but output file $lCaseTmpFile not found.\n";
    }
  }



  if ($lDoFoundLineTest eq "true") {
    my $lPoints_Found = 1 * $lMaxPoints;
    
    $gMaxFoundLinePoints += $lPoints_Found;
    if (CheckFoundLine($lCaseTmpFile, $lCaseAnswerFile) == 1) {
      $lPoints += $lPoints_Found;
    }
  }

  if ($lDoValidLinkTraversalTest eq "true") {
    my $lPoints_ValidLinkTraversalTest = 1 * $lMaxPoints;
    
    $gMaxValidLinkTraversalPoints += $lPoints_ValidLinkTraversalTest;
    my $lTmpPoints = CheckValidLinkTraversal($lCaseTmpFile, $lCaseAnswerFile);
    $lPoints += $lPoints_ValidLinkTraversalTest * $lTmpPoints;
  }

  if ($lDoIgnoredLinkTraversalTest eq "true") {
    my $lPoints_IgnoredLinkTraversalTest = 1 * $lMaxPoints;
    
    $gMaxIgnoredLinkTraversalPoints += $lPoints_IgnoredLinkTraversalTest;
    my $lTmpPoints = CheckIgnoredLinkTraversal($lCaseTmpFile, $lCaseAnswerFile);
    $lPoints += $lPoints_IgnoredLinkTraversalTest * $lTmpPoints;
  }

  if ($lOutputDir ne "") {
    my $lTestMaxPoints = 1 * $lMaxPoints;
    $gMaxSavedOutputPoints += $lTestMaxPoints;
    my $lTmpFraction = CheckSavedOutput($lTestOutputDir, $lOutputDir);
    print "****$lTmpFraction\n";
    $lPoints += $lTestMaxPoints * $lTmpFraction;
    
  }

  if ($lPoints > $lMaxPoints) {
    die "Max Points ($lMaxPoints) exceeded ($lPoints) for Case $lCaseFile\n";
  }

  print "\n\n";
  print "\$\$\$ TestResult: $lCaseFile: $lPoints (out of $lMaxPoints)\n";

  print "\n";
  print "Your output:\n";
  print "******************************\n";
  DumpFile ($lCaseTmpFile);
  print "******************************\n";
  print "\n\n"; 
  print "Expected output:\n";
  print "******************************\n";
  DumpFile($lCaseAnswerFile);
  print "******************************\n";

  print "\n\n";
  print ">>> Finished Running Case : $lCaseFile\n";
  print "-----------------------------------------------------------------------------\n";

  return ($lPoints, $lMaxPoints);
}

sub CheckSavedOutput() {
  my $lTestDir = shift(@_);
  my $lAnsDir = shift(@_);

  my $lTotalDiffs = 0;
  my $lCorrectDiffs = 0;

  my @lAnsOutputFiles = <$lAnsDir/*.html>;
  my $lAnsOutputFile;
  foreach $lAnsOutputFile (@lAnsOutputFiles) {
    my $lDiffCmd = "diff $lAnsOutputFile $lTestDir/".basename($lAnsOutputFile);
    `$lDiffCmd`;
    if ($? == 0) {
      $lCorrectDiffs++;
    }
    $lTotalDiffs++;
  }

  return ($lCorrectDiffs/$lTotalDiffs);
}

sub CheckValidLinkTraversal() {
  my $lTestFile = shift(@_);
  my $lAnsFile = shift(@_);

  open (TESTFD, "$lTestFile") ||die "Unable to open file $lTestFile\n";
  open (ANSFD, "$lAnsFile") ||die "Unable to open file $lAnsFile\n";

  my @lTestLines = <TESTFD>;

  my $lFollowedLinkCountAns = 0;
  my $lFollowedLinkCountTest = 0;

  while (<ANSFD>) {
    my $lCurrLine = $_;
      if ($lCurrLine =~ /action=follow$/) {
        $lFollowedLinkCountAns += 1;
        my $lTmpTestLine;
        foreach $lTmpTestLine (@lTestLines) {
          if ($lTmpTestLine eq $lCurrLine) {
            $lFollowedLinkCountTest += 1;
          }
        }
      }
  }
  
  close (ANSFD);
  close (TESTFD);
  return $lFollowedLinkCountTest/$lFollowedLinkCountAns;

}

sub CheckIgnoredLinkTraversal() {
  my $lTestFile = shift(@_);
  my $lAnsFile = shift(@_);

  open (TESTFD, "$lTestFile") ||die "Unable to open file $lTestFile\n";
  open (ANSFD, "$lAnsFile") ||die "Unable to open file $lAnsFile\n";


  my %lIgnoredLineCountHashAns;
  my %lIgnoredLineCountHashTest;
  
  while (<ANSFD>) {
    my $lCurrLine = $_;
    chomp($lCurrLine);
      if ($lCurrLine =~ /action=already_seen_ignore$/) {
        my $lTmpCount = $lIgnoredLineCountHashAns{$lCurrLine};
        if ($lTmpCount == undef) {
          $lIgnoredLineCountHashAns{$lCurrLine} = 1;
        } else {
          $lIgnoredLineCountHashAns{$lCurrLine} = $lTmpCount + 1;
        }
      }
  }
 
  while (<TESTFD>) {
    my $lCurrLine = $_;
      if ($lCurrLine =~ /action=already_seen_ignore$/) {
        chomp($lCurrLine);
        my $lTmpCount = $lIgnoredLineCountHashTest{$lCurrLine};
        if ($lTmpCount == undef) {
          $lIgnoredLineCountHashTest{$lCurrLine} = 1;
        } else {
          $lIgnoredLineCountHashTest{$lCurrLine} = $lTmpCount + 1;
        }
      }
  }
 
  my $lTmpKey;
  my $lAnsCount = 0;
  my $lTestCount = 0;

  foreach $lTmpKey (keys %lIgnoredLineCountHashAns) {
    $lAnsCount ++;

    my $lTmp1 = $lIgnoredLineCountHashAns{$lTmpKey};
    my $lTmp2 = $lIgnoredLineCountHashTest{$lTmpKey};
    if ($lTmp1 == $lTmp2) {
      $lTestCount++;
    }
  }
  close (ANSFD);
  close (TESTFD);
  
  print ">>>>>>>>$lTestCount/$lAnsCount\n";
  return $lTestCount/$lAnsCount;

}

sub CheckFoundLine() {
  my $lTestFile = shift(@_);
  my $lAnsFile = shift(@_);

  my $lAnsLine;

  open (AnsFd, "$lAnsFile") || die "Unable to open $lAnsFile for reading.\n";
  while(<AnsFd>) {
    if (/^FOUND_KEYWORD:/) {
      $lAnsLine = $_;
      last;
    } elsif (/^NOT_FOUND_KEYWORD:/) {
      $lAnsLine = $_;
    }
  }
  if (!defined($lAnsLine)) {
    die "CheckFoundLine: Invalid answer file!\n";
  }
  close (AnsFd);

  my $lFoundFlag = 0;
  open (TestFd, "$lTestFile") || die "Unable to open $lTestFile for reading.\n";
  while(<TestFd>) {
    if(/^$lAnsLine/) {
      $lFoundFlag = 1;
      last;
    }
  }
  close (TestFd);

  return $lFoundFlag;
  
}

sub DumpFile() {
  my $lFileToDump = shift(@_);
  open (DUMPFD, "$lFileToDump") || die "Unable to open file $lFileToDump for dumping\n";
  while(<DUMPFD>) {
    print $_;
  }
  close (DUMPFD);
}
